//www.HomoFaciens.de/technics-physical-computing-digital-ruler_ge.htm
//
//  Circuit:
//  LCD RS pin to digital pin     11
//  LCD Enable pin to digital pin 10
//  LCD D4 pin to digital pin      9
//  LCD D5 pin to digital pin      8
//  LCD D6 pin to digital pin      7
//  LCD D7 pin to digital pin      6
//  LCD R/W pin to ground
//  10K potentiometer:
//  ends to +5V and ground
//  wiper to LCD VO pin (pin 3)



// include the library code:
#include <LiquidCrystal.h>

//Define pins -------------
#define LCD_RS               11
#define LCD_E                10
#define LCD_D4                9
#define LCD_D5                8
#define LCD_D6                7
#define LCD_D7                6

#define PUSH_BUTTON_L        A0
#define PUSH_BUTTON_R        A1
#define SENSOR_A              3
#define SENSOR_B              2

//Define numbers -------------

//How to get the calibration factor:
//1.) Set CALIBRATION to 1 and load sketch to Arduino
//2.) Move ruler for 400mm
//3.) Read number of pulses
//4.) CALIBRATION = 40 / pulses
#define CALIBRATION     0.12422

#define KEY_DOWN_TIME            500
#define KEY_DOWN_TIME_FAST       100
#define KEY_DOWN_TIME_VERY_FAST   20
#define BOUNCETIME               100

//Define variables -------------
long pulseCounter = 0;
unsigned long switchTimerL = 0;
unsigned long switchTimerR = 0;
byte switchOldStateL = 1;
byte switchOldStateR = 1;
byte keyRepeat = 0;
long keyTimer = BOUNCETIME;
byte sensorStatusA = 0;
byte sensorStatusB = 0;
byte stepStatus = 0;
byte oldStepStatus = 0;
double mmLength = 0;
char LCDtext[40];


//Initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);



void change_sensor_A(){
  //Code is executed whenever the signal at SENSOR_A changes
  //from LOW to HIGH or from HIGH to LOW.
  //Keep the code of that subroutine as compact as possible

  sensorStatusA ^= 0b00000001;
  if(sensorStatusB){
    if(sensorStatusA){
      pulseCounter--;
    }
    else{
      pulseCounter++;
    }
  }
  else{
    if(sensorStatusA){
      pulseCounter++;
    }
    else{
      pulseCounter--;
    }
  }
}

void change_sensor_B(){
  //Code is executed whenever the signal at SENSOR_B changes
  //from LOW to HIGH or from HIGH to LOW.
  //Keep the code of that subroutine as compact as possible

  sensorStatusB ^= 0b00000001;

  if(sensorStatusA){
    if(sensorStatusB){
      pulseCounter++;
    }
    else{
      pulseCounter--;
    }
  }
  else{
    if(sensorStatusB){
      pulseCounter--;
    }
    else{
      pulseCounter++;
    }
  }

}



void setup() {
  pinMode(PUSH_BUTTON_L, INPUT_PULLUP);
  pinMode(PUSH_BUTTON_R, INPUT_PULLUP);

  //Activate hardware interrupts on pin 2 and 3 (sensor pins)
  //Execute sub change_sensor_A whenever signal changes
  //from LOW to HIGH or from HIGH to LOW on pin 2
  attachInterrupt(0, change_sensor_A, CHANGE);
  //Execute sub change_sensor_B whenever signal changes
  //from LOW to HIGH or from HIGH to LOW on pin 3
  attachInterrupt(1, change_sensor_B, CHANGE);

  sensorStatusA = digitalRead(SENSOR_A);
  sensorStatusB = digitalRead(SENSOR_B);

  if(sensorStatusA == 0 && sensorStatusB == 0){
    stepStatus = 0;
  }
  if(sensorStatusA == 1 && sensorStatusB == 0){
    stepStatus = 1;
  }
  if(sensorStatusA == 1 && sensorStatusB == 1){
    stepStatus = 2;
  }
  if(sensorStatusA == 0 && sensorStatusB == 1){
    stepStatus = 3;
  }

  pulseCounter = 0;
  //set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  //Set cursor to row 0, column 0
  lcd.setCursor(0, 0);
  lcd.print("Digital Ruler   ");
  //Set cursor to row 1, column 0
  lcd.setCursor(0, 1);
  lcd.print("HomoFaciens.de  ");
  delay(2000);

  lcd.setCursor(0, 1);
  lcd.print("Length =        ");
  switchTimerL = 0;
  switchTimerR = 0;
  switchOldStateL = 1;
  switchOldStateR = 1;
  keyTimer = BOUNCETIME;
  keyRepeat = 0;
}

void loop() {

  //Set counter to zero whenever both buttons are pressed
  if(!digitalRead(PUSH_BUTTON_L) && !digitalRead(PUSH_BUTTON_R)){
    pulseCounter = 0;
  }

  //Count up whenever the left button is pressed
  if(!digitalRead(PUSH_BUTTON_L)){
    if(switchOldStateL == 1){
      switchTimerL = millis();
      switchOldStateL = 0;
    }
    //If button is pressed for a long time, repeat counting up
    if(millis() - switchTimerL > keyTimer){
      pulseCounter++;
      keyRepeat++;
      switchTimerL = millis();
      if(keyTimer == BOUNCETIME){
       keyTimer = KEY_DOWN_TIME;
      }
    }
    //If button is pressed for a very long time, increase speed of count up
    if(keyRepeat > 5){
      keyTimer = KEY_DOWN_TIME_FAST;
    }
    if(keyRepeat > 100){
      keyTimer = KEY_DOWN_TIME_VERY_FAST;
      keyRepeat = 101;
    }
  }
  else{//Buton is released
    if(switchOldStateL == 0){
      switchTimerL = millis();
      switchOldStateL = 1;
      keyTimer = BOUNCETIME;
      keyRepeat = 0;
    }
  }

  //Count down whenever the right button is pressed
  if(!digitalRead(PUSH_BUTTON_R)){
    if(switchOldStateR == 1){
      switchTimerR = millis();
      switchOldStateR = 0;
    }
    //If button is pressed for a long time, repeat counting down
    if(millis() - switchTimerR > keyTimer){
      pulseCounter--;
      keyRepeat++;
      switchTimerR = millis();
      if(keyTimer == BOUNCETIME){
       keyTimer = KEY_DOWN_TIME;
      }
    }
    //If button is pressed for a very long time, increase speed of count down
    if(keyRepeat > 5){
      keyTimer = KEY_DOWN_TIME_FAST;
    }
    if(keyRepeat > 100){
      keyTimer = KEY_DOWN_TIME_VERY_FAST;
      keyRepeat = 101;
    }
  }
  else{//Buton is released
    if(switchOldStateR == 0){
      switchTimerR = millis();
      switchOldStateR = 1;
      keyTimer = BOUNCETIME;
      keyRepeat = 0;
    }
  }


  //Calculate length in mm and display result
  mmLength = pulseCounter * CALIBRATION;
  lcd.setCursor(9, 1);
  if(mmLength > -1){
    lcd.print(" ");
  }
  lcd.setCursor(9, 1);
  if(mmLength >= 0){
    lcd.print(" ");
  }
  if(abs(mmLength) < 10){
    lcd.print(" ");
  }
  if(abs(mmLength) < 100){
    lcd.print(" ");
  }
  if(abs(mmLength) < 1000){
    lcd.print(" ");
  }
  lcd.print(mmLength);
  lcd.print(" ");

  //lcd.print(LCDtext);
  lcd.print(LCDtext);
  //delay(50);
}


